# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneauth1.exceptions import http
from keystoneclient.tests.functional import base
from keystoneclient.tests.functional.v3 import client_fixtures as fixtures


class UsersTestCase(base.V3ClientTestCase):

    def check_user(self, user, user_ref=None):
        self.assertIsNotNone(user.id)
        self.assertIsNotNone(user.enabled)
        self.assertIn('self', user.links)
        self.assertIn('/users/' + user.id, user.links['self'])

        if user_ref:
            self.assertEqual(user_ref['name'], user.name)
            self.assertEqual(user_ref['domain'], user.domain_id)
            # There is no guarantee the attributes below are present in user
            if hasattr(user_ref, 'description'):
                self.assertEqual(user_ref['description'], user.description)
            if hasattr(user_ref, 'email'):
                self.assertEqual(user_ref['email'], user.email)
            if hasattr(user_ref, 'default_project'):
                self.assertEqual(user_ref['default_project'],
                                 user.default_project_id)
        else:
            # Only check remaining mandatory attributes
            self.assertIsNotNone(user.name)
            self.assertIsNotNone(user.domain_id)

    def test_create_user(self):
        user_ref = {
            'name': fixtures.RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
            'domain': self.project_domain_id,
            'default_project': self.project_id,
            'password': uuid.uuid4().hex,
            'description': uuid.uuid4().hex}

        user = self.client.users.create(**user_ref)
        self.addCleanup(self.client.users.delete, user)
        self.check_user(user, user_ref)

    def test_get_user(self):
        user = fixtures.User(self.client, self.project_domain_id)
        self.useFixture(user)

        user_ret = self.client.users.get(user.id)
        self.check_user(user_ret, user.ref)

    def test_list_users(self):
        user_one = fixtures.User(self.client, self.project_domain_id)
        self.useFixture(user_one)

        user_two = fixtures.User(self.client, self.project_domain_id)
        self.useFixture(user_two)

        users = self.client.users.list()

        # All users are valid
        for user in users:
            self.check_user(user)

        self.assertIn(user_one.entity, users)
        self.assertIn(user_two.entity, users)

    def test_list_users_with_filters(self):
        suffix = uuid.uuid4().hex
        user1_ref = {
            'name': 'test_user' + suffix,
            'domain': self.project_domain_id,
            'default_project': self.project_id,
            'password': uuid.uuid4().hex,
            'description': uuid.uuid4().hex}

        user2_ref = {
            'name': fixtures.RESOURCE_NAME_PREFIX + uuid.uuid4().hex,
            'domain': self.project_domain_id,
            'default_project': self.project_id,
            'password': uuid.uuid4().hex,
            'description': uuid.uuid4().hex}

        user1 = self.client.users.create(**user1_ref)
        self.client.users.create(**user2_ref)

        users = self.client.users.list(name__contains=['test_user', suffix])
        self.assertEqual(1, len(users))
        self.assertIn(user1, users)

    def test_update_user(self):
        user = fixtures.User(self.client, self.project_domain_id)
        self.useFixture(user)

        new_description = uuid.uuid4().hex
        user_ret = self.client.users.update(user.id,
                                            description=new_description)

        user.ref.update({'description': new_description})
        self.check_user(user_ret, user.ref)

    def test_user_grouping(self):
        # keystoneclient.v3.users owns user grouping operations, this is why
        # this test case belongs to this class
        user = fixtures.User(self.client, self.project_domain_id)
        group = fixtures.Group(self.client, self.project_domain_id)
        self.useFixture(user)
        self.useFixture(group)

        self.assertRaises(http.NotFound,
                          self.client.users.check_in_group,
                          user.id, group.id)

        self.client.users.add_to_group(user.id, group.id)
        self.client.users.check_in_group(user.id, group.id)
        self.client.users.remove_from_group(user.id, group.id)

        self.assertRaises(http.NotFound,
                          self.client.users.check_in_group,
                          user.id, group.id)

    def test_delete_user(self):
        user = self.client.users.create(name=uuid.uuid4().hex,
                                        domain=self.project_domain_id)

        self.client.users.delete(user.id)
        self.assertRaises(http.NotFound,
                          self.client.users.get,
                          user.id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base


class Protocol(base.Resource):
    """An object representing federation protocol container.

    Attributes:
        * id: user-defined unique per Identity Provider string identifying
              federation protocol.

    """

    pass


class ProtocolManager(base.CrudManager):
    """Manager class for manipulating federation protocols."""

    resource_class = Protocol
    collection_key = 'protocols'
    key = 'protocol'
    base_url = 'OS-FEDERATION/identity_providers'

    def build_url(self, dict_args_in_out=None):
        """Build URL for federation protocols."""
        if dict_args_in_out is None:
            dict_args_in_out = {}

        identity_provider_id = dict_args_in_out.pop('identity_provider_id',
                                                    None)
        if identity_provider_id:
            base_url = '/'.join([self.base_url, identity_provider_id])
        else:
            base_url = self.base_url

        dict_args_in_out.setdefault('base_url', base_url)
        return super(ProtocolManager, self).build_url(dict_args_in_out)

    def _build_url_and_put(self, request_body=None, **kwargs):
        url = self.build_url(dict_args_in_out=kwargs)
        body = {self.key: request_body}
        return self._update(url, body=body,
                            response_key=self.key,
                            method='PUT')

    def create(self, protocol_id, identity_provider, mapping, **kwargs):
        """Create federation protocol object and tie to the Identity Provider.

        Utilize Identity API operation:
        PUT /OS-FEDERATION/identity_providers/
        $identity_provider/protocols/$protocol

        :param protocol_id: a string type parameter identifying a federation
            protocol
        :param identity_provider: a string type parameter identifying an
            Identity Provider
        :param mapping: a base.Resource object with federation mapping id

        """
        return self._build_url_and_put(
            request_body={'mapping_id': base.getid(mapping)},
            identity_provider_id=base.getid(identity_provider),
            protocol_id=protocol_id, **kwargs)

    def get(self, identity_provider, protocol, **kwargs):
        """Fetch federation protocol object tied to the Identity Provider.

        Utilize Identity API operation:
        GET /OS-FEDERATION/identity_providers/
        $identity_provider/protocols/$protocol

        :param identity_provider: a base.Resource type object with Identity
                                  Provider id stored inside
        :param protocol: a base.Resource type object with federation protocol
                         id stored inside

        """
        return super(ProtocolManager, self).get(
            identity_provider_id=base.getid(identity_provider),
            protocol_id=base.getid(protocol), **kwargs)

    def list(self, identity_provider, **kwargs):
        """List all federation protocol objects tied to the Identity Provider.

        Utilize Identity API operation:
        GET /OS-FEDERATION/identity_providers/
        $identity_provider/protocols

        :param identity_provider: a base.Resource type object with Identity
            Provider id stored inside

        """
        return super(ProtocolManager, self).list(
            identity_provider_id=base.getid(identity_provider), **kwargs)

    def update(self, identity_provider, protocol, mapping, **kwargs):
        """Update Protocol object tied to the Identity Provider.

        Utilize Identity API operation:
        PATCH /OS-FEDERATION/identity_providers/
        $identity_provider/protocols/$protocol

        :param identity_provider: a base.Resource type object with Identity
            Provider id stored inside
        :param protocol: a base.Resource type object with federation protocol
            id stored inside
        :param mapping: a base.Resource object with federation mapping id


        """
        return super(ProtocolManager, self).update(
            identity_provider_id=base.getid(identity_provider),
            protocol_id=base.getid(protocol), mapping_id=base.getid(mapping),
            **kwargs)

    def delete(self, identity_provider, protocol):
        """Delete Protocol object tied to the Identity Provider.

        Utilize Identity API operation:
        DELETE /OS-FEDERATION/identity_providers/
        $identity_provider/protocols/$protocol

        :param identity_provider: a base.Resource type object with
            Identity Provider id stored inside
        :param protocol: a base.Resource type object with federation
            protocol id stored inside

        """
        return super(ProtocolManager, self).delete(
            identity_provider_id=base.getid(identity_provider),
            protocol_id=base.getid(protocol))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from debtcollector import removals
from keystoneauth1 import plugin
import stevedore

from keystoneclient import exceptions


# NOTE(jamielennox): The AUTH_INTERFACE is a special value that can be
# requested from get_endpoint. If a plugin receives this as the value of
# 'interface' it should return the initial URL that was passed to the plugin.
AUTH_INTERFACE = plugin.AUTH_INTERFACE

PLUGIN_NAMESPACE = 'keystoneclient.auth.plugin'
IDENTITY_AUTH_HEADER_NAME = 'X-Auth-Token'


@removals.remove(
    message='keystoneclient auth plugins are deprecated. Use keystoneauth.',
    version='2.1.0',
    removal_version='3.0.0'
)
def get_available_plugin_names():
    """Get the names of all the plugins that are available on the system.

    This is particularly useful for help and error text to prompt a user for
    example what plugins they may specify.

    :returns: A list of names.
    :rtype: frozenset
    """
    mgr = stevedore.ExtensionManager(namespace=PLUGIN_NAMESPACE,
                                     invoke_on_load=False)
    return frozenset(mgr.names())


@removals.remove(
    message='keystoneclient auth plugins are deprecated. Use keystoneauth.',
    version='2.1.0',
    removal_version='3.0.0'
)
def get_available_plugin_classes():
    """Retrieve all the plugin classes available on the system.

    :returns: A dict with plugin entrypoint name as the key and the plugin
              class as the value.
    :rtype: dict
    """
    mgr = stevedore.ExtensionManager(namespace=PLUGIN_NAMESPACE,
                                     propagate_map_exceptions=True,
                                     invoke_on_load=False)

    return dict(mgr.map(lambda ext: (ext.entry_point.name, ext.plugin)))


@removals.remove(
    message='keystoneclient auth plugins are deprecated. Use keystoneauth.',
    version='2.1.0',
    removal_version='3.0.0'
)
def get_plugin_class(name):
    """Retrieve a plugin class by its entrypoint name.

    :param str name: The name of the object to get.

    :returns: An auth plugin class.
    :rtype: :py:class:`keystoneclient.auth.BaseAuthPlugin`

    :raises keystoneclient.exceptions.NoMatchingPlugin: if a plugin cannot be
                                                        created.
    """
    try:
        mgr = stevedore.DriverManager(namespace=PLUGIN_NAMESPACE,
                                      name=name,
                                      invoke_on_load=False)
    except RuntimeError:
        raise exceptions.NoMatchingPlugin(name)

    return mgr.driver


class BaseAuthPlugin(object):
    """The basic structure of an authentication plugin."""

    def get_token(self, session, **kwargs):
        """Obtain a token.

        How the token is obtained is up to the plugin. If it is still valid
        it may be re-used, retrieved from cache or invoke an authentication
        request against a server.

        There are no required kwargs. They are passed directly to the auth
        plugin and they are implementation specific.

        Returning None will indicate that no token was able to be retrieved.

        This function is misplaced as it should only be required for auth
        plugins that use the 'X-Auth-Token' header. However due to the way
        plugins evolved this method is required and often called to trigger an
        authentication request on a new plugin.

        When implementing a new plugin it is advised that you implement this
        method, however if you don't require the 'X-Auth-Token' header override
        the `get_headers` method instead.

        :param session: A session object so the plugin can make HTTP calls.
        :type session: keystoneclient.session.Session

        :return: A token to use.
        :rtype: string
        """
        return None

    def get_headers(self, session, **kwargs):
        """Fetch authentication headers for message.

        This is a more generalized replacement of the older get_token to allow
        plugins to specify different or additional authentication headers to
        the OpenStack standard 'X-Auth-Token' header.

        How the authentication headers are obtained is up to the plugin. If the
        headers are still valid they may be re-used, retrieved from cache or
        the plugin may invoke an authentication request against a server.

        The default implementation of get_headers calls the `get_token` method
        to enable older style plugins to continue functioning unchanged.
        Subclasses should feel free to completely override this function to
        provide the headers that they want.

        There are no required kwargs. They are passed directly to the auth
        plugin and they are implementation specific.

        Returning None will indicate that no token was able to be retrieved and
        that authorization was a failure. Adding no authentication data can be
        achieved by returning an empty dictionary.

        :param session: The session object that the auth_plugin belongs to.
        :type session: keystoneclient.session.Session

        :returns: Headers that are set to authenticate a message or None for
                  failure. Note that when checking this value that the empty
                  dict is a valid, non-failure response.
        :rtype: dict
        """
        token = self.get_token(session)

        if not token:
            return None

        return {IDENTITY_AUTH_HEADER_NAME: token}

    def get_endpoint(self, session, **kwargs):
        """Return an endpoint for the client.

        There are no required keyword arguments to ``get_endpoint`` as a plugin
        implementation should use best effort with the information available to
        determine the endpoint. However there are certain standard options that
        will be generated by the clients and should be used by plugins:

        - ``service_type``: what sort of service is required.
        - ``service_name``: the name of the service in the catalog.
        - ``interface``: what visibility the endpoint should have.
        - ``region_name``: the region the endpoint exists in.

        :param session: The session object that the auth_plugin belongs to.
        :type session: keystoneclient.session.Session

        :returns: The base URL that will be used to talk to the required
                  service or None if not available.
        :rtype: string
        """
        return None

    def get_connection_params(self, session, **kwargs):
        """Return any additional connection parameters required for the plugin.

        :param session: The session object that the auth_plugin belongs to.
        :type session: keystoneclient.session.Session

        :returns: Headers that are set to authenticate a message or None for
                  failure. Note that when checking this value that the empty
                  dict is a valid, non-failure response.
        :rtype: dict
        """
        return {}

    def invalidate(self):
        """Invalidate the current authentication data.

        This should result in fetching a new token on next call.

        A plugin may be invalidated if an Unauthorized HTTP response is
        returned to indicate that the token may have been revoked or is
        otherwise now invalid.

        :returns: True if there was something that the plugin did to
                  invalidate. This means that it makes sense to try again. If
                  nothing happens returns False to indicate give up.
        :rtype: bool
        """
        return False

    def get_user_id(self, session, **kwargs):
        """Return a unique user identifier of the plugin.

        Wherever possible the user id should be inferred from the token however
        there are certain URLs and other places that require access to the
        currently authenticated user id.

        :param session: A session object so the plugin can make HTTP calls.
        :type session: keystoneclient.session.Session

        :returns: A user identifier or None if one is not available.
        :rtype: str
        """
        return None

    def get_project_id(self, session, **kwargs):
        """Return the project id that we are authenticated to.

        Wherever possible the project id should be inferred from the token
        however there are certain URLs and other places that require access to
        the currently authenticated project id.

        :param session: A session object so the plugin can make HTTP calls.
        :type session: keystoneclient.session.Session

        :returns: A project identifier or None if one is not available.
        :rtype: str
        """
        return None

    @classmethod
    def get_options(cls):
        """Return the list of parameters associated with the auth plugin.

        This list may be used to generate CLI or config arguments.

        :returns: A list of Param objects describing available plugin
                  parameters.
        :rtype: List
        """
        return []

    @classmethod
    def load_from_options(cls, **kwargs):
        """Create a plugin from the arguments retrieved from get_options.

        A client can override this function to do argument validation or to
        handle differences between the registered options and what is required
        to create the plugin.
        """
        return cls(**kwargs)

    @classmethod
    def register_argparse_arguments(cls, parser):
        """Register the CLI options provided by a specific plugin.

        Given a plugin class convert it's options into argparse arguments and
        add them to a parser.

        :param parser: the parser to attach argparse options.
        :type parser: argparse.ArgumentParser
        """
        # NOTE(jamielennox): ideally oslo_config would be smart enough to
        # handle all the Opt manipulation that goes on in this file. However it
        # is currently not.  Options are handled in as similar a way as
        # possible to oslo_config such that when available we should be able to
        # transition.

        for opt in cls.get_options():
            args = []
            envs = []

            for o in [opt] + opt.deprecated_opts:
                args.append('--os-%s' % o.name)
                envs.append('OS_%s' % o.name.replace('-', '_').upper())

            # select the first ENV that is not false-y or return None
            env_vars = (os.environ.get(e) for e in envs)
            default = next(filter(None, env_vars), None)

            parser.add_argument(*args,
                                default=default or opt.default,
                                metavar=opt.metavar,
                                help=opt.help,
                                dest='os_%s' % opt.dest)

    @classmethod
    def load_from_argparse_arguments(cls, namespace, **kwargs):
        """Load a specific plugin object from an argparse result.

        Convert the results of a parse into the specified plugin.

        :param namespace: The result from CLI parsing.
        :type namespace: argparse.Namespace

        :returns: An auth plugin, or None if a name is not provided.
        :rtype: :py:class:`keystoneclient.auth.BaseAuthPlugin`
        """
        def _getter(opt):
            return getattr(namespace, 'os_%s' % opt.dest)

        return cls.load_from_options_getter(_getter, **kwargs)

    @classmethod
    def register_conf_options(cls, conf, group):
        """Register the oslo_config options that are needed for a plugin.

        :param conf: A config object.
        :type conf: oslo_config.cfg.ConfigOpts
        :param string group: The group name that options should be read from.
        """
        plugin_opts = cls.get_options()
        conf.register_opts(plugin_opts, group=group)

    @classmethod
    def load_from_conf_options(cls, conf, group, **kwargs):
        """Load the plugin from a CONF object.

        Convert the options already registered into a real plugin.

        :param conf: A config object.
        :type conf: oslo_config.cfg.ConfigOpts
        :param string group: The group name that options should be read from.

        :returns: An authentication Plugin.
        :rtype: :py:class:`keystoneclient.auth.BaseAuthPlugin`
        """
        def _getter(opt):
            return conf[group][opt.dest]

        return cls.load_from_options_getter(_getter, **kwargs)

    @classmethod
    def load_from_options_getter(cls, getter, **kwargs):
        """Load a plugin from a getter function returning appropriate values.

        To handle cases other than the provided CONF and CLI loading you can
        specify a custom loader function that will be queried for the option
        value.

        The getter is a function that takes one value, an
        :py:class:`oslo_config.cfg.Opt` and returns a value to load with.

        :param getter: A function that returns a value for the given opt.
        :type getter: callable

        :returns: An authentication Plugin.
        :rtype: :py:class:`keystoneclient.auth.BaseAuthPlugin`
        """
        plugin_opts = cls.get_options()

        for opt in plugin_opts:
            val = getter(opt)
            if val is not None:
                val = opt.type(val)
            kwargs.setdefault(opt.dest, val)

        return cls.load_from_options(**kwargs)



# Copyright 2011 OpenStack Foundation
# Copyright 2011 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from debtcollector import renames

from keystoneclient import base
from keystoneclient import exceptions
from keystoneclient.i18n import _


class User(base.Resource):
    """Represents an Identity user.

    Attributes:
        * id: a uuid that identifies the user

    """

    pass


class UserManager(base.CrudManager):
    """Manager class for manipulating Identity users."""

    resource_class = User
    collection_key = 'users'
    key = 'user'

    def _require_user_and_group(self, user, group):
        if not (user and group):
            msg = _('Specify both a user and a group')
            raise exceptions.ValidationError(msg)

    @renames.renamed_kwarg('project', 'default_project', version='1.7.0',
                           removal_version='2.0.0')
    def create(self, name, domain=None, project=None, password=None,
               email=None, description=None, enabled=True,
               default_project=None, **kwargs):
        """Create a user.

        :param str name: the name of the user.
        :param domain: the domain of the user.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param project: the default project of the user.
                        (deprecated, see warning below)
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param str password: the password for the user.
        :param str email: the email address of the user.
        :param str description: a description of the user.
        :param bool enabled: whether the user is enabled.
        :param default_project: the default project of the user.
        :type default_project: str or
                               :class:`keystoneclient.v3.projects.Project`
        :param kwargs: any other attribute provided will be passed to the
                       server.

        :returns: the created user returned from server.
        :rtype: :class:`keystoneclient.v3.users.User`

        .. warning::

          The project argument is deprecated as of the 1.7.0 release in favor
          of default_project and may be removed in the 2.0.0 release.

          If both default_project and project is provided, the default_project
          will be used.

        """
        default_project_id = base.getid(default_project) or base.getid(project)
        user_data = base.filter_none(name=name,
                                     domain_id=base.getid(domain),
                                     default_project_id=default_project_id,
                                     password=password,
                                     email=email,
                                     description=description,
                                     enabled=enabled,
                                     **kwargs)

        return self._post('/users', {'user': user_data}, 'user',
                          log=not bool(password))

    @renames.renamed_kwarg('project', 'default_project', version='1.7.0',
                           removal_version='2.0.0')
    def list(self, project=None, domain=None, group=None, default_project=None,
             **kwargs):
        """List users.

        :param project: the default project of the users to be filtered on.
                        (deprecated, see warning below)
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param domain: the domain of the users to be filtered on.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param group: the group in which the users are member of.
        :type group: str or :class:`keystoneclient.v3.groups.Group`
        :param default_project: the default project of the users to be filtered
                                on.
        :type default_project: str or
                               :class:`keystoneclient.v3.projects.Project`
        :param kwargs: any other attribute provided will filter users on.

        :returns: a list of users.
        :rtype: list of :class:`keystoneclient.v3.users.User`.

        .. warning::

          The project argument is deprecated as of the 1.7.0 release in favor
          of default_project and may be removed in the 2.0.0 release.

          If both default_project and project is provided, the default_project
          will be used.

        """
        default_project_id = base.getid(default_project) or base.getid(project)
        if group:
            base_url = '/groups/%s' % base.getid(group)
        else:
            base_url = None

        return super(UserManager, self).list(
            base_url=base_url,
            domain_id=base.getid(domain),
            default_project_id=default_project_id,
            **kwargs)

    def get(self, user):
        """Retrieve a user.

        :param user: the user to be retrieved from the server.
        :type user: str or :class:`keystoneclient.v3.users.User`

        :returns: the specified user returned from server.
        :rtype: :class:`keystoneclient.v3.users.User`

        """
        return super(UserManager, self).get(
            user_id=base.getid(user))

    @renames.renamed_kwarg('project', 'default_project', version='1.7.0',
                           removal_version='2.0.0')
    def update(self, user, name=None, domain=None, project=None, password=None,
               email=None, description=None, enabled=None,
               default_project=None, **kwargs):
        """Update a user.

        :param user: the user to be updated on the server.
        :type user: str or :class:`keystoneclient.v3.users.User`
        :param str name: the new name of the user.
        :param domain: the new domain of the user.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param project: the new default project of the user.
                        (deprecated, see warning below)
        :type project: str or :class:`keystoneclient.v3.projects.Project`
        :param str password: the new password of the user.
        :param str email: the new email of the user.
        :param str description: the newdescription of the user.
        :param bool enabled: whether the user is enabled.
        :param default_project: the new default project of the user.
        :type default_project: str or
                               :class:`keystoneclient.v3.projects.Project`
        :param kwargs: any other attribute provided will be passed to server.

        :returns: the updated user returned from server.
        :rtype: :class:`keystoneclient.v3.users.User`

        .. warning::

          The project argument is deprecated as of the 1.7.0 release in favor
          of default_project and may be removed in the 2.0.0 release.

          If both default_project and project is provided, the default_project
          will be used.

        """
        default_project_id = base.getid(default_project) or base.getid(project)
        user_data = base.filter_none(name=name,
                                     domain_id=base.getid(domain),
                                     default_project_id=default_project_id,
                                     password=password,
                                     email=email,
                                     description=description,
                                     enabled=enabled,
                                     **kwargs)

        return self._update('/users/%s' % base.getid(user),
                            {'user': user_data},
                            'user',
                            method='PATCH',
                            log=False)

    def update_password(self, old_password, new_password):
        """Update the password for the user the token belongs to.

        :param str old_password: the user's old password
        :param str new_password: the user's new password

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        if not (old_password and new_password):
            msg = _('Specify both the current password and a new password')
            raise exceptions.ValidationError(msg)

        if old_password == new_password:
            msg = _('Old password and new password must be different.')
            raise exceptions.ValidationError(msg)

        params = {'user': {'password': new_password,
                           'original_password': old_password}}

        base_url = '/users/%s/password' % self.client.user_id

        return self._update(base_url, params, method='POST', log=False)

    def add_to_group(self, user, group):
        """Add the specified user as a member of the specified group.

        :param user: the user to be added to the group.
        :type user: str or :class:`keystoneclient.v3.users.User`
        :param group: the group to put the user in.
        :type group: str or :class:`keystoneclient.v3.groups.Group`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        self._require_user_and_group(user, group)

        base_url = '/groups/%s' % base.getid(group)
        return super(UserManager, self).put(
            base_url=base_url,
            user_id=base.getid(user))

    def check_in_group(self, user, group):
        """Check if the specified user is a member of the specified group.

        :param user: the user to be verified in the group.
        :type user: str or :class:`keystoneclient.v3.users.User`
        :param group: the group to check the user in.
        :type group: str or :class:`keystoneclient.v3.groups.Group`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        self._require_user_and_group(user, group)

        base_url = '/groups/%s' % base.getid(group)
        return super(UserManager, self).head(
            base_url=base_url,
            user_id=base.getid(user))

    def remove_from_group(self, user, group):
        """Remove the specified user from the specified group.

        :param user: the user to be removed from the group.
        :type user: str or :class:`keystoneclient.v3.users.User`
        :param group: the group to remove the user from.
        :type group: str or :class:`keystoneclient.v3.groups.Group`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        self._require_user_and_group(user, group)

        base_url = '/groups/%s' % base.getid(group)
        return super(UserManager, self).delete(
            base_url=base_url,
            user_id=base.getid(user))

    def delete(self, user):
        """Delete a user.

        :param user: the user to be deleted on the server.
        :type user: str or :class:`keystoneclient.v3.users.User`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        return super(UserManager, self).delete(
            user_id=base.getid(user))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base
from keystoneclient import exceptions
from keystoneclient.i18n import _


class RoleAssignment(base.Resource):
    """Represents an Identity role assignment.

    Attributes:
        * role: an object which contains a role uuid
        * user or group: an object which contains either a user or
                         group uuid
        * scope: an object which has either a project or domain object
                 containing an uuid
    """

    pass


class RoleAssignmentManager(base.CrudManager):
    """Manager class for manipulating Identity roles assignments."""

    resource_class = RoleAssignment
    collection_key = 'role_assignments'
    key = 'role_assignment'

    def _check_not_user_and_group(self, user, group):
        if user and group:
            msg = _('Specify either a user or group, not both')
            raise exceptions.ValidationError(msg)

    def _check_not_domain_and_project(self, domain, project):
        if domain and project:
            msg = _('Specify either a domain or project, not both')
            raise exceptions.ValidationError(msg)

    def _check_not_system_and_domain(self, system, domain):
        if system and domain:
            msg = _('Specify either system or domain, not both')
            raise exceptions.ValidationError(msg)

    def _check_not_system_and_project(self, system, project):
        if system and project:
            msg = _('Specify either system or project, not both')
            raise exceptions.ValidationError(msg)

    def _check_system_value(self, system):
        if system and system != 'all':
            msg = _("Only a system scope of 'all' is currently supported")
            raise exceptions.ValidationError(msg)

    def list(self, user=None, group=None, project=None, domain=None,
             system=False, role=None, effective=False,
             os_inherit_extension_inherited_to=None, include_subtree=False,
             include_names=False):
        """List role assignments.

        If no arguments are provided, all role assignments in the
        system will be listed.

        If both user and group are provided, a ValidationError will be
        raised. If both domain and project are provided, it will also
        raise a ValidationError.

        :param user: User to be used as query filter. (optional)
        :param group: Group to be used as query filter. (optional)
        :param project: Project to be used as query filter.
                        (optional)
        :param domain: Domain to be used as query
                       filter. (optional)
        :param system: Boolean to be used to filter system assignments.
                       (optional)
        :param role: Role to be used as query filter. (optional)
        :param boolean effective: return effective role
                                  assignments. (optional)
        :param string os_inherit_extension_inherited_to:
            return inherited role assignments for either 'projects' or
            'domains'. (optional)
        :param boolean include_subtree: Include subtree (optional)
        :param boolean include_names: Display names instead
                                      of IDs. (optional)
        """
        self._check_not_user_and_group(user, group)
        self._check_not_domain_and_project(domain, project)
        self._check_not_system_and_domain(system, domain)
        self._check_not_system_and_project(system, project)
        self._check_system_value(system)

        query_params = {}
        if user:
            query_params['user.id'] = base.getid(user)
        if group:
            query_params['group.id'] = base.getid(group)
        if project:
            query_params['scope.project.id'] = base.getid(project)
        if domain:
            query_params['scope.domain.id'] = base.getid(domain)
        if system:
            query_params['scope.system'] = system
        if role:
            query_params['role.id'] = base.getid(role)
        if effective:
            query_params['effective'] = effective
        if include_names:
            query_params['include_names'] = include_names
        if os_inherit_extension_inherited_to:
            query_params['scope.OS-INHERIT:inherited_to'] = (
                os_inherit_extension_inherited_to)
        if include_subtree:
            query_params['include_subtree'] = include_subtree

        return super(RoleAssignmentManager, self).list(**query_params)

    def create(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('Create not supported for role assignments'))

    def update(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('Update not supported for role assignments'))

    def get(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('Get not supported for role assignments'))

    def find(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('Find not supported for role assignments'))

    def put(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('Put not supported for role assignments'))

    def delete(self, **kwargs):
        raise exceptions.MethodNotImplemented(
            _('Delete not supported for role assignments'))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import warnings

from debtcollector import removals
from keystoneauth1 import plugin

from keystoneclient import _discover
from keystoneclient import exceptions
from keystoneclient.i18n import _
from keystoneclient import session as client_session
from keystoneclient.v2_0 import client as v2_client
from keystoneclient.v3 import client as v3_client


_CLIENT_VERSIONS = {2: v2_client.Client,
                    3: v3_client.Client}


# functions needed from the private file that can be made public

def normalize_version_number(version):
    """Turn a version representation into a tuple.

    Takes a string, tuple or float which represent version formats we can
    handle and converts them into a (major, minor) version tuple that we can
    actually use for discovery.

    e.g. 'v3.3' gives (3, 3)
         3.1 gives (3, 1)

    :param version: Inputted version number to try and convert.

    :returns: A usable version tuple
    :rtype: tuple

    :raises TypeError: if the inputted version cannot be converted to tuple.
    """
    return _discover.normalize_version_number(version)


def version_match(required, candidate):
    """Test that an available version satisfies the required version.

    To be suitable a version must be of the same major version as required
    and be at least a match in minor/patch level.

    eg. 3.3 is a match for a required 3.1 but 4.1 is not.

    :param tuple required: the version that must be met.
    :param tuple candidate: the version to test against required.

    :returns: True if candidate is suitable False otherwise.
    :rtype: bool
    """
    return _discover.version_match(required, candidate)


def available_versions(url, session=None, **kwargs):
    """Retrieve raw version data from a url."""
    if not session:
        session = client_session.Session._construct(kwargs)

    return _discover.get_version_data(session, url)


class Discover(_discover.Discover):
    """A means to discover and create clients.

    Clients are created depending on the supported API versions on the server.

    Querying the server is done on object creation and every subsequent method
    operates upon the data that was retrieved.

    The connection parameters associated with this method are the same format
    and name as those used by a client (see
    :py:class:`keystoneclient.v2_0.client.Client` and
    :py:class:`keystoneclient.v3.client.Client`). If not overridden in
    subsequent methods they will also be what is passed to the constructed
    client.

    In the event that auth_url and endpoint is provided then auth_url will be
    used in accordance with how the client operates.

    .. warning::

        Creating an instance of this class without using the session argument
        is deprecated as of the 1.7.0 release and may be removed in the 2.0.0
        release.

    :param session: A session object that will be used for communication.
                    Clients will also be constructed with this session.
    :type session: keystoneclient.session.Session
    :param string auth_url: Identity service endpoint for authorization.
                            (optional)
    :param string endpoint: A user-supplied endpoint URL for the identity
                            service. (optional)
    :param string original_ip: The original IP of the requesting user which
                               will be sent to identity service in a
                               'Forwarded' header. (optional) This is ignored
                               if a session is provided. Deprecated as of the
                               1.7.0 release and may be removed in the 2.0.0
                               release.
    :param boolean debug: Enables debug logging of all request and responses to
                          the identity service. default False (optional)
                          This is ignored if a session is provided. Deprecated
                          as of the 1.7.0 release and may be removed in the
                          2.0.0 release.
    :param string cacert: Path to the Privacy Enhanced Mail (PEM) file which
                          contains the trusted authority X.509 certificates
                          needed to established SSL connection with the
                          identity service. (optional) This is ignored if a
                          session is provided. Deprecated as of the 1.7.0
                          release and may be removed in the 2.0.0 release.
    :param string key: Path to the Privacy Enhanced Mail (PEM) file which
                       contains the unencrypted client private key needed to
                       established two-way SSL connection with the identity
                       service. (optional) This is ignored if a session is
                       provided. Deprecated as of the 1.7.0 release and may be
                       removed in the 2.0.0 release.
    :param string cert: Path to the Privacy Enhanced Mail (PEM) file which
                        contains the corresponding X.509 client certificate
                        needed to established two-way SSL connection with the
                        identity service. (optional) This is ignored if a
                        session is provided. Deprecated as of the 1.7.0 release
                        and may be removed in the 2.0.0 release.
    :param boolean insecure: Does not perform X.509 certificate validation when
                             establishing SSL connection with identity service.
                             default: False (optional) This is ignored if a
                             session is provided. Deprecated as of the 1.7.0
                             release and may be removed in the 2.0.0 release.
    :param bool authenticated: Should a token be used to perform the initial
                               discovery operations. default: None (attach a
                               token if an auth plugin is available).

    """

    def __init__(self, session=None, authenticated=None, **kwargs):
        if not session:
            warnings.warn(
                'Constructing a Discover instance without using a session is '
                'deprecated as of the 1.7.0 release and may be removed in the '
                '2.0.0 release.', DeprecationWarning)
            session = client_session.Session._construct(kwargs)
        kwargs['session'] = session

        url = None
        endpoint = kwargs.pop('endpoint', None)
        auth_url = kwargs.pop('auth_url', None)

        if endpoint:
            self._use_endpoint = True
            url = endpoint
        elif auth_url:
            self._use_endpoint = False
            url = auth_url
        elif session.auth:
            self._use_endpoint = False
            url = session.get_endpoint(interface=plugin.AUTH_INTERFACE)

        if not url:
            raise exceptions.DiscoveryFailure(
                _('Not enough information to determine URL. Provide'
                  ' either a Session, or auth_url or endpoint'))

        self._client_kwargs = kwargs
        super(Discover, self).__init__(session, url,
                                       authenticated=authenticated)

    @removals.remove(message='Use raw_version_data instead.', version='1.7.0',
                     removal_version='2.0.0')
    def available_versions(self, **kwargs):
        """Return a list of identity APIs available on the server.

        The list returned includes the data associated with them.

        .. warning::

            This method is deprecated as of the 1.7.0 release in favor of
            :meth:`raw_version_data` and may be removed in the 2.0.0 release.

        :param bool unstable: Accept endpoints not marked 'stable'. (optional)
                              Equates to setting allow_experimental
                              and allow_unknown to True.
        :param bool allow_experimental: Allow experimental version endpoints.
        :param bool allow_deprecated: Allow deprecated version endpoints.
        :param bool allow_unknown: Allow endpoints with an unrecognised status.

        :returns: A List of dictionaries as presented by the server. Each dict
                  will contain the version and the URL to use for the version.
                  It is a direct representation of the layout presented by the
                  identity API.
        """
        return self.raw_version_data(**kwargs)

    @removals.removed_kwarg(
        'unstable',
        message='Use allow_experimental and allow_unknown instead.',
        version='1.7.0', removal_version='2.0.0')
    def raw_version_data(self, unstable=False, **kwargs):
        """Get raw version information from URL.

        Raw data indicates that only minimal validation processing is performed
        on the data, so what is returned here will be the data in the same
        format it was received from the endpoint.

        :param bool unstable: equates to setting allow_experimental and
                              allow_unknown. This argument is deprecated as of
                              the 1.7.0 release and may be removed in the 2.0.0
                              release.
        :param bool allow_experimental: Allow experimental version endpoints.
        :param bool allow_deprecated: Allow deprecated version endpoints.
        :param bool allow_unknown: Allow endpoints with an unrecognised status.

        :returns: The endpoints returned from the server that match the
                  criteria.
        :rtype: List

        Example::

            >>> from keystoneclient import discover
            >>> disc = discover.Discovery(auth_url='http://localhost:5000')
            >>> disc.raw_version_data()
                [{'id': 'v3.0',
                    'links': [{'href': 'http://127.0.0.1:5000/v3/',
                               'rel': 'self'}],
                  'media-types': [
                      {'base': 'application/json',
                       'type': 'application/vnd.openstack.identity-v3+json'},
                      {'base': 'application/xml',
                       'type': 'application/vnd.openstack.identity-v3+xml'}],
                  'status': 'stable',
                  'updated': '2013-03-06T00:00:00Z'},
                 {'id': 'v2.0',
                  'links': [{'href': 'http://127.0.0.1:5000/v2.0/',
                             'rel': 'self'},
                            {'href': '...',
                             'rel': 'describedby',
                             'type': 'application/pdf'}],
                  'media-types': [
                      {'base': 'application/json',
                       'type': 'application/vnd.openstack.identity-v2.0+json'},
                      {'base': 'application/xml',
                       'type': 'application/vnd.openstack.identity-v2.0+xml'}],
                  'status': 'stable',
                  'updated': '2013-03-06T00:00:00Z'}]
        """
        if unstable:
            kwargs.setdefault('allow_experimental', True)
            kwargs.setdefault('allow_unknown', True)

        return super(Discover, self).raw_version_data(**kwargs)

    def _calculate_version(self, version, unstable):
        version_data = None

        if version:
            version_data = self.data_for(version)
        else:
            # if no version specified pick the latest one
            all_versions = self.version_data(unstable=unstable)
            if all_versions:
                version_data = all_versions[-1]

        if not version_data:
            msg = _('Could not find a suitable endpoint')

            if version:
                msg = _('Could not find a suitable endpoint for client '
                        'version: %s') % str(version)

            raise exceptions.VersionNotAvailable(msg)

        return version_data

    def _create_client(self, version_data, **kwargs):
        # Get the client for the version requested that was returned
        try:
            client_class = _CLIENT_VERSIONS[version_data['version'][0]]
        except KeyError:
            version = '.'.join(str(v) for v in version_data['version'])
            msg = _('No client available for version: %s') % version
            raise exceptions.DiscoveryFailure(msg)

        # kwargs should take priority over stored kwargs.
        for k, v in self._client_kwargs.items():
            kwargs.setdefault(k, v)

        # restore the url to either auth_url or endpoint depending on what
        # was initially given
        if self._use_endpoint:
            kwargs['auth_url'] = None
            kwargs['endpoint'] = version_data['url']
        else:
            kwargs['auth_url'] = version_data['url']
            kwargs['endpoint'] = None

        return client_class(**kwargs)

    def create_client(self, version=None, unstable=False, **kwargs):
        """Factory function to create a new identity service client.

        :param tuple version: The required version of the identity API. If
                              specified the client will be selected such that
                              the major version is equivalent and an endpoint
                              provides at least the specified minor version.
                              For example to specify the 3.1 API use (3, 1).
                              (optional)
        :param bool unstable: Accept endpoints not marked 'stable'. (optional)
        :param kwargs: Additional arguments will override those provided to
                       this object's constructor.

        :returns: An instantiated identity client object.

        :raises keystoneclient.exceptions.DiscoveryFailure: if the server
                                                            response is invalid
        :raises keystoneclient.exceptions.VersionNotAvailable: if a suitable
            client cannot be found.
        """
        version_data = self._calculate_version(version, unstable)
        return self._create_client(version_data, **kwargs)


def add_catalog_discover_hack(service_type, old, new):
    """Add a version removal rule for a particular service.

    Originally deployments of OpenStack would contain a versioned endpoint in
    the catalog for different services. E.g. an identity service might look
    like ``http://localhost:5000/v2.0``. This is a problem when we want to use
    a different version like v3.0 as there is no way to tell where it is
    located. We cannot simply change all service catalogs either so there must
    be a way to handle the older style of catalog.

    This function adds a rule for a given service type that if part of the URL
    matches a given regular expression in *old* then it will be replaced with
    the *new* value. This will replace all instances of old with new. It should
    therefore contain a regex anchor.

    For example the included rule states::

        add_catalog_version_hack('identity', re.compile('/v2.0/?$'), '/')

    so if the catalog retrieves an *identity* URL that ends with /v2.0 or
    /v2.0/ then it should replace it simply with / to fix the user's catalog.

    :param str service_type: The service type as defined in the catalog that
                             the rule will apply to.
    :param re.RegexObject old: The regular expression to search for and replace
                               if found.
    :param str new: The new string to replace the pattern with.
    """
    _discover._VERSION_HACKS.add_discover_hack(service_type, old, new)



# Copyright 2011 OpenStack Foundation
# Copyright 2011 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base


class Domain(base.Resource):
    """Represents an Identity domain.

    Attributes:
        * id: a uuid that identifies the domain
        * name: the name of the domain
        * description: a description of the domain
        * enabled: determines whether the domain is enabled

    """

    pass


class DomainManager(base.CrudManager):
    """Manager class for manipulating Identity domains."""

    resource_class = Domain
    collection_key = 'domains'
    key = 'domain'

    def create(self, name, description=None, enabled=True, **kwargs):
        """Create a domain.

        :param str name: the name of the domain.
        :param str description: a description of the domain.
        :param bool enabled: whether the domain is enabled.
        :param kwargs: any other attribute provided will be passed to the
                       server.

        :returns: the created domain returned from server.
        :rtype: :class:`keystoneclient.v3.domains.Domain`

        """
        return super(DomainManager, self).create(
            name=name,
            description=description,
            enabled=enabled,
            **kwargs)

    def get(self, domain):
        """Retrieve a domain.

        :param domain: the domain to be retrieved from the server.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`

        :returns: the specified domain returned from server.
        :rtype: :class:`keystoneclient.v3.domains.Domain`

        """
        return super(DomainManager, self).get(
            domain_id=base.getid(domain))

    def list(self, **kwargs):
        """List domains.

        :param kwargs: allows filter criteria to be passed where
         supported by the server.

        :returns: a list of domains.
        :rtype: list of :class:`keystoneclient.v3.domains.Domain`.

        """
        # Ref bug #1267530 we have to pass 0 for False to get the expected
        # results on all keystone versions
        if kwargs.get('enabled') is False:
            kwargs['enabled'] = 0
        return super(DomainManager, self).list(**kwargs)

    def update(self, domain, name=None,
               description=None, enabled=None, **kwargs):
        """Update a domain.

        :param domain: the domain to be updated on the server.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`
        :param str name: the new name of the domain.
        :param str description: the new description of the domain.
        :param bool enabled: whether the domain is enabled.
        :param kwargs: any other attribute provided will be passed to the
                       server.

        :returns: the updated domain returned from server.
        :rtype: :class:`keystoneclient.v3.domains.Domain`

        """
        return super(DomainManager, self).update(
            domain_id=base.getid(domain),
            name=name,
            description=description,
            enabled=enabled,
            **kwargs)

    def delete(self, domain):
        """Delete a domain.

        :param domain: the domain to be deleted on the server.
        :type domain: str or :class:`keystoneclient.v3.domains.Domain`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        return super(DomainManager, self).delete(
            domain_id=base.getid(domain))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import warnings

from oslo_serialization import jsonutils


class Adapter(object):
    """An instance of a session with local variables.

    A session is a global object that is shared around amongst many clients. It
    therefore contains state that is relevant to everyone. There is a lot of
    state such as the service type and region_name that are only relevant to a
    particular client that is using the session. An adapter provides a wrapper
    of client local data around the global session object.

    :param session: The session object to wrap.
    :type session: keystoneclient.session.Session
    :param str service_type: The default service_type for URL discovery.
    :param str service_name: The default service_name for URL discovery.
    :param str interface: The default interface for URL discovery.
    :param str region_name: The default region_name for URL discovery.
    :param str endpoint_override: Always use this endpoint URL for requests
                                  for this client.
    :param tuple version: The version that this API targets.
    :param auth: An auth plugin to use instead of the session one.
    :type auth: keystoneclient.auth.base.BaseAuthPlugin
    :param str user_agent: The User-Agent string to set.
    :param int connect_retries: the maximum number of retries that should
                                be attempted for connection errors.
                                Default None - use session default which
                                is don't retry.
    :param logger: A logging object to use for requests that pass through this
                   adapter.
    :type logger: logging.Logger
    """

    def __init__(self, session, service_type=None, service_name=None,
                 interface=None, region_name=None, endpoint_override=None,
                 version=None, auth=None, user_agent=None,
                 connect_retries=None, logger=None):
        warnings.warn(
            'keystoneclient.adapter.Adapter is deprecated as of the 2.1.0 '
            'release in favor of keystoneauth1.adapter.Adapter. It will be '
            'removed in future releases.', DeprecationWarning)

        # NOTE(jamielennox): when adding new parameters to adapter please also
        # add them to the adapter call in httpclient.HTTPClient.__init__
        self.session = session
        self.service_type = service_type
        self.service_name = service_name
        self.interface = interface
        self.region_name = region_name
        self.endpoint_override = endpoint_override
        self.version = version
        self.user_agent = user_agent
        self.auth = auth
        self.connect_retries = connect_retries
        self.logger = logger

    def _set_endpoint_filter_kwargs(self, kwargs):
        if self.service_type:
            kwargs.setdefault('service_type', self.service_type)
        if self.service_name:
            kwargs.setdefault('service_name', self.service_name)
        if self.interface:
            kwargs.setdefault('interface', self.interface)
        if self.region_name:
            kwargs.setdefault('region_name', self.region_name)
        if self.version:
            kwargs.setdefault('version', self.version)

    def request(self, url, method, **kwargs):
        endpoint_filter = kwargs.setdefault('endpoint_filter', {})
        self._set_endpoint_filter_kwargs(endpoint_filter)

        if self.endpoint_override:
            kwargs.setdefault('endpoint_override', self.endpoint_override)

        if self.auth:
            kwargs.setdefault('auth', self.auth)
        if self.user_agent:
            kwargs.setdefault('user_agent', self.user_agent)
        if self.connect_retries is not None:
            kwargs.setdefault('connect_retries', self.connect_retries)
        if self.logger:
            kwargs.setdefault('logger', self.logger)

        return self.session.request(url, method, **kwargs)

    def get_token(self, auth=None):
        """Return a token as provided by the auth plugin.

        :param auth: The auth plugin to use for token. Overrides the plugin
                     on the session. (optional)
        :type auth: :class:`keystoneclient.auth.base.BaseAuthPlugin`

        :raises keystoneclient.exceptions.AuthorizationFailure: if a new token
            fetch fails.

        :returns: A valid token.
        :rtype: string
        """
        return self.session.get_token(auth or self.auth)

    def get_endpoint(self, auth=None, **kwargs):
        """Get an endpoint as provided by the auth plugin.

        :param auth: The auth plugin to use for token. Overrides the plugin on
                     the session. (optional)
        :type auth: :class:`keystoneclient.auth.base.BaseAuthPlugin`

        :raises keystoneclient.exceptions.MissingAuthPlugin: if a plugin is not
                                                             available.

        :returns: An endpoint if available or None.
        :rtype: string
        """
        if self.endpoint_override:
            return self.endpoint_override

        self._set_endpoint_filter_kwargs(kwargs)
        return self.session.get_endpoint(auth or self.auth, **kwargs)

    def invalidate(self, auth=None):
        """Invalidate an authentication plugin."""
        return self.session.invalidate(auth or self.auth)

    def get_user_id(self, auth=None):
        """Return the authenticated user_id as provided by the auth plugin.

        :param auth: The auth plugin to use for token. Overrides the plugin
                     on the session. (optional)
        :type auth: keystoneclient.auth.base.BaseAuthPlugin

        :raises keystoneclient.exceptions.AuthorizationFailure:
            if a new token fetch fails.
        :raises keystoneclient.exceptions.MissingAuthPlugin:
            if a plugin is not available.

        :returns: Current `user_id` or None if not supported by plugin.
        :rtype: string
        """
        return self.session.get_user_id(auth or self.auth)

    def get_project_id(self, auth=None):
        """Return the authenticated project_id as provided by the auth plugin.

        :param auth: The auth plugin to use for token. Overrides the plugin
                     on the session. (optional)
        :type auth: keystoneclient.auth.base.BaseAuthPlugin

        :raises keystoneclient.exceptions.AuthorizationFailure:
            if a new token fetch fails.
        :raises keystoneclient.exceptions.MissingAuthPlugin:
            if a plugin is not available.

        :returns: Current `project_id` or None if not supported by plugin.
        :rtype: string
        """
        return self.session.get_project_id(auth or self.auth)

    def get(self, url, **kwargs):
        return self.request(url, 'GET', **kwargs)

    def head(self, url, **kwargs):
        return self.request(url, 'HEAD', **kwargs)

    def post(self, url, **kwargs):
        return self.request(url, 'POST', **kwargs)

    def put(self, url, **kwargs):
        return self.request(url, 'PUT', **kwargs)

    def patch(self, url, **kwargs):
        return self.request(url, 'PATCH', **kwargs)

    def delete(self, url, **kwargs):
        return self.request(url, 'DELETE', **kwargs)


class LegacyJsonAdapter(Adapter):
    """Make something that looks like an old HTTPClient.

    A common case when using an adapter is that we want an interface similar to
    the HTTPClients of old which returned the body as JSON as well.

    You probably don't want this if you are starting from scratch.
    """

    def request(self, *args, **kwargs):
        headers = kwargs.setdefault('headers', {})
        headers.setdefault('Accept', 'application/json')

        try:
            kwargs['json'] = kwargs.pop('body')
        except KeyError:  # nosec(cjschaef): kwargs doesn't contain a 'body'
            # key, while 'json' is an optional argument for Session.request
            pass

        resp = super(LegacyJsonAdapter, self).request(*args, **kwargs)

        body = None
        if resp.text:
            try:
                body = jsonutils.loads(resp.text)
            except ValueError:  # nosec(cjschaef): return None for body as
                # expected
                pass

        return resp, body



# Copyright 2011 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import warnings

from keystoneclient.auth.identity import v2 as v2_auth
from keystoneclient import exceptions
from keystoneclient import httpclient
from keystoneclient.i18n import _
from keystoneclient.v2_0 import certificates
from keystoneclient.v2_0 import ec2
from keystoneclient.v2_0 import endpoints
from keystoneclient.v2_0 import extensions
from keystoneclient.v2_0 import roles
from keystoneclient.v2_0 import services
from keystoneclient.v2_0 import tenants
from keystoneclient.v2_0 import tokens
from keystoneclient.v2_0 import users


_logger = logging.getLogger(__name__)


class Client(httpclient.HTTPClient):
    """Client for the OpenStack Keystone v2.0 API.

    :param string username: Username for authentication. (optional)
    :param string password: Password for authentication. (optional)
    :param string token: Token for authentication. (optional)
    :param string tenant_id: Tenant id. (optional)
    :param string tenant_name: Tenant name. (optional)
    :param string auth_url: Keystone service endpoint for authorization.
    :param string region_name: Name of a region to select when choosing an
                               endpoint from the service catalog.
    :param string endpoint: A user-supplied endpoint URL for the keystone
                            service.  Lazy-authentication is possible for API
                            service calls if endpoint is set at
                            instantiation.(optional)
    :param integer timeout: Allows customization of the timeout for client
                            http requests. (optional)
    :param string original_ip: The original IP of the requesting user
                               which will be sent to Keystone in a
                               'Forwarded' header. (optional)
    :param string cert: Path to the Privacy Enhanced Mail (PEM) file which
                        contains the corresponding X.509 client certificate
                        needed to established two-way SSL connection with
                        the identity service. (optional)
    :param string key: Path to the Privacy Enhanced Mail (PEM) file which
                       contains the unencrypted client private key needed
                       to established two-way SSL connection with the
                       identity service. (optional)
    :param string cacert: Path to the Privacy Enhanced Mail (PEM) file which
                          contains the trusted authority X.509 certificates
                          needed to established SSL connection with the
                          identity service. (optional)
    :param boolean insecure: Does not perform X.509 certificate validation
                             when establishing SSL connection with identity
                             service. default: False (optional)
    :param dict auth_ref: To allow for consumers of the client to manage their
                          own caching strategy, you may initialize a client
                          with a previously captured auth_reference (token)
    :param boolean debug: Enables debug logging of all request and responses
                          to keystone. default False (option)

    .. warning::

        If debug is enabled, it may show passwords in plain text as a part of
        its output.

    .. warning::

        Constructing an instance of this class without a session is
        deprecated as of the 1.7.0 release and will be removed in the
        2.0.0 release.

    The client can be created and used like a user or in a strictly
    bootstrap mode. Normal operation expects a username, password, auth_url,
    and tenant_name or id to be provided. Other values will be lazily loaded
    as needed from the service catalog.

    Example::

        >>> from keystoneauth1.identity import v2
        >>> from keystoneauth1 import session
        >>> from keystoneclient.v2_0 import client
        >>> auth = v2.Password(auth_url=KEYSTONE_URL,
        ...                    username=USER,
        ...                    password=PASS,
        ...                    tenant_name=TENANT_NAME)
        >>> sess = session.Session(auth=auth)
        >>> keystone = client.Client(session=sess)
        >>> keystone.tenants.list()
        ...
        >>> user = keystone.users.get(USER_ID)
        >>> user.delete()

    Once authenticated, you can store and attempt to re-use the
    authenticated token. the auth_ref property on the client
    returns as a dictionary-like-object so that you can export and
    cache it, re-using it when initiating another client::

        >>> from keystoneauth1.identity import v2
        >>> from keystoneauth1 import session
        >>> from keystoneclient.v2_0 import client
        >>> auth = v2.Password(auth_url=KEYSTONE_URL,
        ...                    username=USER,
        ...                    password=PASS,
        ...                    tenant_name=TENANT_NAME)
        >>> sess = session.Session(auth=auth)
        >>> keystone = client.Client(session=sess)
        >>> auth_ref = keystone.auth_ref
        >>> # pickle or whatever you like here
        >>> new_client = client.Client(auth_ref=auth_ref)

    Alternatively, you can provide the administrative token configured in
    keystone and an endpoint to communicate with directly. See
    (``admin_token`` in ``keystone.conf``) In this case, authenticate()
    is not needed, and no service catalog will be loaded.

    Example::

        >>> from keystoneauth1.identity import v2
        >>> from keystoneauth1 import session
        >>> from keystoneclient.v2_0 import client
        >>> auth = v2.Token(auth_url='http://localhost:35357/v2.0',
        ...                 token='12345secret7890')
        >>> sess = session.Session(auth=auth)
        >>> keystone = client.Client(session=sess)
        >>> keystone.tenants.list()

    """

    version = 'v2.0'

    def __init__(self, **kwargs):
        """Initialize a new client for the Keystone v2.0 API."""
        if not kwargs.get('session'):
            warnings.warn(
                'Constructing an instance of the '
                'keystoneclient.v2_0.client.Client class without a session is '
                'deprecated as of the 1.7.0 release and may be removed in '
                'the 2.0.0 release.', DeprecationWarning)

        super(Client, self).__init__(**kwargs)

        self.certificates = certificates.CertificatesManager(self._adapter)
        self.endpoints = endpoints.EndpointManager(self._adapter)
        self.extensions = extensions.ExtensionManager(self._adapter)
        self.roles = roles.RoleManager(self._adapter)
        self.services = services.ServiceManager(self._adapter)
        self.tokens = tokens.TokenManager(self._adapter)
        self.users = users.UserManager(self._adapter, self.roles)

        self.tenants = tenants.TenantManager(self._adapter,
                                             self.roles, self.users)

        # extensions
        self.ec2 = ec2.CredentialsManager(self._adapter)

        # DEPRECATED: if session is passed then we go to the new behaviour of
        # authenticating on the first required call.
        if not kwargs.get('session') and self.management_url is None:
            self.authenticate()

    def get_raw_token_from_identity_service(self, auth_url, username=None,
                                            password=None, tenant_name=None,
                                            tenant_id=None, token=None,
                                            project_name=None, project_id=None,
                                            trust_id=None,
                                            **kwargs):
        """Authenticate against the v2 Identity API.

        If a token is provided it will be used in preference over username and
        password.

        :returns: access.AccessInfo if authentication was successful.
        :raises keystoneclient.exceptions.AuthorizationFailure: if unable to
            authenticate or validate the existing authorization token
        """
        try:
            if auth_url is None:
                raise ValueError(_("Cannot authenticate without an auth_url"))

            new_kwargs = {'trust_id': trust_id,
                          'tenant_id': project_id or tenant_id,
                          'tenant_name': project_name or tenant_name}

            if token:
                plugin = v2_auth.Token(auth_url, token, **new_kwargs)
            elif username and password:
                plugin = v2_auth.Password(auth_url, username, password,
                                          **new_kwargs)
            else:
                msg = _('A username and password or token is required.')
                raise exceptions.AuthorizationFailure(msg)

            return plugin.get_auth_ref(self.session)
        except (exceptions.AuthorizationFailure, exceptions.Unauthorized):
            _logger.debug("Authorization Failed.")
            raise
        except exceptions.EndpointNotFound:
            msg = (
                _('There was no suitable authentication url for this request'))
            raise exceptions.AuthorizationFailure(msg)
        except Exception as e:
            raise exceptions.AuthorizationFailure(
                _("Authorization Failed: %s") % e)



# Copyright 2011 OpenStack Foundation
# Copyright 2011 Nebula, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneclient import base
from keystoneclient import exceptions
from keystoneclient.i18n import _


VALID_INTERFACES = ['public', 'admin', 'internal']


class Endpoint(base.Resource):
    """Represents an Identity endpoint.

    Attributes:
        * id: a uuid that identifies the endpoint
        * interface: 'public', 'admin' or 'internal' network interface
        * region: geographic location of the endpoint
        * service_id: service to which the endpoint belongs
        * url: fully qualified service endpoint
        * enabled: determines whether the endpoint appears in the service
                   catalog

    """

    pass


class EndpointManager(base.CrudManager):
    """Manager class for manipulating Identity endpoints."""

    resource_class = Endpoint
    collection_key = 'endpoints'
    key = 'endpoint'

    def _validate_interface(self, interface):
        if interface is not None and interface not in VALID_INTERFACES:
            msg = _('"interface" must be one of: %s')
            msg %= ', '.join(VALID_INTERFACES)
            raise exceptions.ValidationError(msg)

    def create(self, service, url, interface=None, region=None, enabled=True,
               **kwargs):
        """Create an endpoint.

        :param service: the service to which the endpoint belongs.
        :type service: str or :class:`keystoneclient.v3.services.Service`
        :param str url: the URL of the fully qualified service endpoint.
        :param str interface: the network interface of the endpoint. Valid
                             values are: ``public``, ``admin`` or ``internal``.
        :param region: the region to which the endpoint belongs.
        :type region: str or :class:`keystoneclient.v3.regions.Region`
        :param bool enabled: whether the endpoint is enabled or not,
                             determining if it appears in the service catalog.
        :param kwargs: any other attribute provided will be passed to the
                       server.

        :returns: the created endpoint returned from server.
        :rtype: :class:`keystoneclient.v3.endpoints.Endpoint`

        """
        self._validate_interface(interface)
        return super(EndpointManager, self).create(
            service_id=base.getid(service),
            interface=interface,
            url=url,
            region=region,
            enabled=enabled,
            **kwargs)

    def get(self, endpoint):
        """Retrieve an endpoint.

        :param endpoint: the endpoint to be retrieved from the server.
        :type endpoint: str or :class:`keystoneclient.v3.endpoints.Endpoint`

        :returns: the specified endpoint returned from server.
        :rtype: :class:`keystoneclient.v3.endpoints.Endpoint`

        """
        return super(EndpointManager, self).get(
            endpoint_id=base.getid(endpoint))

    def list(self, service=None, interface=None, region=None, enabled=None,
             region_id=None, **kwargs):
        """List endpoints.

        :param service: the service of the endpoints to be filtered on.
        :type service: str or :class:`keystoneclient.v3.services.Service`
        :param str interface: the network interface of the endpoints to be
                              filtered on. Valid values are: ``public``,
                              ``admin`` or ``internal``.
        :param bool enabled: whether to return enabled or disabled endpoints.
        :param str region_id: filter endpoints by the region_id attribute. If
                              both region and region_id are specified, region
                              takes precedence.
        :param kwargs: any other attribute provided will filter endpoints on.

        :returns: a list of endpoints.
        :rtype: list of :class:`keystoneclient.v3.endpoints.Endpoint`

        """
        # NOTE(lhcheng): region filter is not supported by keystone,
        # region_id should be used instead. Consider removing the
        # region argument in the next release.
        self._validate_interface(interface)
        return super(EndpointManager, self).list(
            service_id=base.getid(service),
            interface=interface,
            region_id=region_id or base.getid(region),
            enabled=enabled,
            **kwargs)

    def update(self, endpoint, service=None, url=None, interface=None,
               region=None, enabled=None, **kwargs):
        """Update an endpoint.

        :param endpoint: the endpoint to be updated on the server.
        :type endpoint: str or :class:`keystoneclient.v3.endpoints.Endpoint`
        :param service: the new service to which the endpoint belongs.
        :type service: str or :class:`keystoneclient.v3.services.Service`
        :param str url: the new URL of the fully qualified service endpoint.
        :param str interface: the new network interface of the endpoint. Valid
                             values are: ``public``, ``admin`` or ``internal``.
        :param region: the new region to which the endpoint belongs.
        :type region: str or :class:`keystoneclient.v3.regions.Region`
        :param bool enabled: determining if the endpoint appears in the service
                             catalog by enabling or disabling it.
        :param kwargs: any other attribute provided will be passed to the
                       server.

        :returns: the updated endpoint returned from server.
        :rtype: :class:`keystoneclient.v3.endpoints.Endpoint`

        """
        self._validate_interface(interface)
        return super(EndpointManager, self).update(
            endpoint_id=base.getid(endpoint),
            service_id=base.getid(service),
            interface=interface,
            url=url,
            region=region,
            enabled=enabled,
            **kwargs)

    def delete(self, endpoint):
        """Delete an endpoint.

        :param endpoint: the endpoint to be deleted on the server.
        :type endpoint: str or :class:`keystoneclient.v3.endpoints.Endpoint`

        :returns: Response object with 204 status.
        :rtype: :class:`requests.models.Response`

        """
        return super(EndpointManager, self).delete(
            endpoint_id=base.getid(endpoint))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneclient.auth.identity.generic import token
from keystoneclient.auth.identity import v2
from keystoneclient.auth.identity import v3
from keystoneclient.auth.identity.v3 import token as v3_token
from keystoneclient.tests.unit.auth import utils


class TokenTests(utils.GenericPluginTestCase):

    PLUGIN_CLASS = token.Token
    V2_PLUGIN_CLASS = v2.Token
    V3_PLUGIN_CLASS = v3.Token

    def new_plugin(self, **kwargs):
        kwargs.setdefault('token', uuid.uuid4().hex)
        return super(TokenTests, self).new_plugin(**kwargs)

    def test_options(self):
        opts = [o.name for o in self.PLUGIN_CLASS.get_options()]

        allowed_opts = ['token',
                        'domain-id',
                        'domain-name',
                        'tenant-id',
                        'tenant-name',
                        'project-id',
                        'project-name',
                        'project-domain-id',
                        'project-domain-name',
                        'trust-id',
                        'auth-url']

        self.assertEqual(set(allowed_opts), set(opts))
        self.assertEqual(len(allowed_opts), len(opts))

    def test_symbols(self):
        self.assertIs(v3.Token, v3_token.Token)
        self.assertIs(v3.TokenMethod, v3_token.TokenMethod)



# Copyright 2012 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid

from keystoneclient.tests.unit.v3 import utils
from keystoneclient.v3 import groups


class GroupTests(utils.ClientTestCase, utils.CrudTests):
    def setUp(self):
        super(GroupTests, self).setUp()
        self.key = 'group'
        self.collection_key = 'groups'
        self.model = groups.Group
        self.manager = self.client.groups

    def new_ref(self, **kwargs):
        kwargs = super(GroupTests, self).new_ref(**kwargs)
        kwargs.setdefault('name', uuid.uuid4().hex)
        return kwargs

    def test_list_groups_for_user(self):
        user_id = uuid.uuid4().hex
        ref_list = [self.new_ref(), self.new_ref()]

        self.stub_entity('GET',
                         ['users', user_id, self.collection_key],
                         status_code=200, entity=ref_list)

        returned_list = self.manager.list(user=user_id)
        self.assertEqual(len(ref_list), len(returned_list))
        [self.assertIsInstance(r, self.model) for r in returned_list]

    def test_list_groups_for_domain(self):
        ref_list = [self.new_ref(), self.new_ref()]
        domain_id = uuid.uuid4().hex

        self.stub_entity('GET',
                         [self.collection_key],
                         status_code=200, entity=ref_list)

        returned_list = self.manager.list(domain=domain_id)
        self.assertTrue(len(ref_list), len(returned_list))
        [self.assertIsInstance(r, self.model) for r in returned_list]

        self.assertQueryStringIs('domain_id=%s' % domain_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneclient import access
from keystoneclient import base


def _calc_id(token):
    if isinstance(token, access.AccessInfo):
        return token.auth_token

    return base.getid(token)


class TokenManager(object):
    """Manager class for manipulating Identity tokens."""

    def __init__(self, client):
        self._client = client

    def revoke_token(self, token):
        """Revoke a token.

        :param token: The token to be revoked.
        :type token: str or :class:`keystoneclient.access.AccessInfo`

        """
        token_id = _calc_id(token)
        headers = {'X-Subject-Token': token_id}
        return self._client.delete('/auth/tokens', headers=headers)

    def get_revoked(self, audit_id_only=False):
        """Get revoked tokens list.

        :param bool audit_id_only: If true, the server is requested to not send
                                   token IDs, but only audit IDs instead.
                                   **New in version 2.2.0.**
        :returns: A dict containing ``signed`` which is a CMS formatted string
                  if the server signed the response. If `audit_id_only` is true
                  then the response may be a dict containing ``revoked`` which
                  is the list of token audit IDs and expiration times.
        :rtype: dict

        """
        path = '/auth/tokens/OS-PKI/revoked'
        if audit_id_only:
            path += '?audit_id_only'
        resp, body = self._client.get(path)
        return body

    def get_token_data(self, token, include_catalog=True, allow_expired=False,
                       access_rules_support=None):
        """Fetch the data about a token from the identity server.

        :param str token: The ID of the token to be fetched.
        :param bool include_catalog: Whether the service catalog should be
                                     included in the response.
        :param allow_expired: If True the token will be validated and returned
                              if it has already expired.
        :param access_rules_support: Version number indicating that the client
                                     is capable of enforcing keystone
                                     access rules, if unset this client
                                     does not support access rules.
        :type access_rules_support: float

        :rtype: dict

        """
        headers = {'X-Subject-Token': token}
        if access_rules_support:
            headers['OpenStack-Identity-Access-Rules'] = access_rules_support
        flags = []

        url = '/auth/tokens'

        if not include_catalog:
            flags.append('nocatalog')
        if allow_expired:
            flags.append('allow_expired=1')

        if flags:
            url = '%s?%s' % (url, '&'.join(flags))

        resp, body = self._client.get(url, headers=headers)
        return body

    def validate(self, token, include_catalog=True, allow_expired=False,
                 access_rules_support=None):
        """Validate a token.

        :param token: The token to be validated.
        :type token: str or :class:`keystoneclient.access.AccessInfo`
        :param include_catalog: If False, the response is requested to not
                                include the catalog.
        :param allow_expired: If True the token will be validated and returned
                              if it has already expired.
        :type allow_expired: bool
        :param access_rules_support: Version number indicating that the client
                                     is capable of enforcing keystone
                                     access rules, if unset this client
                                     does not support access rules.
        :type access_rules_support: float

        :rtype: :class:`keystoneclient.access.AccessInfoV3`

        """
        token_id = _calc_id(token)
        body = self.get_token_data(token_id,
                                   include_catalog=include_catalog,
                                   allow_expired=allow_expired,
                                   access_rules_support=access_rules_support)
        return access.AccessInfo.factory(auth_token=token_id, body=body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import argparse
from unittest import mock
import uuid


from keystoneclient.auth.identity.generic import password
from keystoneclient.auth.identity import v2
from keystoneclient.auth.identity import v3
from keystoneclient.auth.identity.v3 import password as v3_password
from keystoneclient.tests.unit.auth import utils


class PasswordTests(utils.GenericPluginTestCase):

    PLUGIN_CLASS = password.Password
    V2_PLUGIN_CLASS = v2.Password
    V3_PLUGIN_CLASS = v3.Password

    def new_plugin(self, **kwargs):
        kwargs.setdefault('username', uuid.uuid4().hex)
        kwargs.setdefault('password', uuid.uuid4().hex)
        return super(PasswordTests, self).new_plugin(**kwargs)

    def test_with_user_domain_params(self):
        self.stub_discovery()

        self.assertCreateV3(domain_id=uuid.uuid4().hex,
                            user_domain_id=uuid.uuid4().hex)

    def test_v3_user_params_v2_url(self):
        self.stub_discovery(v3=False)
        self.assertDiscoveryFailure(user_domain_id=uuid.uuid4().hex)

    def test_options(self):
        opts = [o.name for o in self.PLUGIN_CLASS.get_options()]

        allowed_opts = ['username',
                        'user-domain-id',
                        'user-domain-name',
                        'user-id',
                        'password',

                        'domain-id',
                        'domain-name',
                        'tenant-id',
                        'tenant-name',
                        'project-id',
                        'project-name',
                        'project-domain-id',
                        'project-domain-name',
                        'trust-id',
                        'auth-url']

        self.assertEqual(set(allowed_opts), set(opts))
        self.assertEqual(len(allowed_opts), len(opts))

    def test_symbols(self):
        self.assertIs(v3.Password, v3_password.Password)
        self.assertIs(v3.PasswordMethod, v3_password.PasswordMethod)

    @mock.patch('sys.stdin', autospec=True)
    def test_prompt_password(self, mock_stdin):
        parser = argparse.ArgumentParser()
        self.PLUGIN_CLASS.register_argparse_arguments(parser)

        username = uuid.uuid4().hex
        user_domain_id = uuid.uuid4().hex
        auth_url = uuid.uuid4().hex
        project_id = uuid.uuid4().hex
        password = uuid.uuid4().hex

        opts = parser.parse_args(['--os-username', username,
                                  '--os-auth-url', auth_url,
                                  '--os-user-domain-id', user_domain_id,
                                  '--os-project-id', project_id])

        with mock.patch('getpass.getpass') as mock_getpass:
            mock_getpass.return_value = password
            mock_stdin.isatty = lambda: True

            plugin = self.PLUGIN_CLASS.load_from_argparse_arguments(opts)

            self.assertEqual(auth_url, plugin.auth_url)
            self.assertEqual(username, plugin._username)
            self.assertEqual(project_id, plugin._project_id)
            self.assertEqual(user_domain_id, plugin._user_domain_id)
            self.assertEqual(password, plugin._password)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from keystoneauth1 import exceptions
from keystoneauth1 import plugin

from keystoneclient import base


class EntityManager(base.Manager, metaclass=abc.ABCMeta):
    """Manager class for listing federated accessible objects."""

    resource_class = None

    @property
    @abc.abstractmethod
    def object_type(self):
        raise exceptions.MethodNotImplemented

    def list(self):
        url = '/auth/%s' % self.object_type
        try:
            tenant_list = self._list(url, self.object_type)
        except exceptions.CatalogException:
            endpoint_filter = {'interface': plugin.AUTH_INTERFACE}
            tenant_list = self._list(url, self.object_type,
                                     endpoint_filter=endpoint_filter)
        return tenant_list
